( cd usr/bin ; rm -rf test3Dfx )
( cd usr/bin ; ln -sf testGlide2x test3Dfx )
( cd usr/lib ; rm -rf libglide.so )
( cd usr/lib ; ln -sf libglide.so.2 libglide.so )
( cd usr/lib ; rm -rf libglide.so.2 )
( cd usr/lib ; ln -sf libglide.so.2.46 libglide.so.2 )
( cd usr/lib ; rm -rf libglide2x.so )
( cd usr/lib ; ln -sf libglide2x.so.2 libglide2x.so )
( cd usr/lib ; rm -rf libglide2x.so.2 )
( cd usr/lib ; ln -sf libglide.so.2.46 libglide2x.so.2 )
